@@ -32,7 +32,6 @@ __all__ = [
 
 from datetime import datetime, date
 from email.utils import formatdate
-import cgi
 import copy
 import os
 import simplejson
@@ -48,6 +47,11 @@ except ImportError:
      import sha
      sha_constructor = sha.new
 
+try:
+    from html import escape
+except ImportError:
+    from cgi import escape
+
 import six
 from zope.component import (
     adapter,
@@ -174,7 +178,7 @@ def encode_value(value):
 
 def _default_html_renderer(value):
     """The default technique for rendering a value as an HTML snippet."""
-    return cgi.escape(six.text_type(value))
+    return escape(six.text_type(value), quote=False)
 
 
 @adapter(Interface, IField, IWebServiceClientRequest)
@@ -442,7 +446,7 @@ class HTTPResource:
 
         # Append the media type, so that web browsers won't treat
         # different representations of a resource interchangeably.
-        core_hashes[-1].update("\0" + media_type)
+        core_hashes[-1].update(("\0" + media_type).encode("UTF-8"))
 
         etag = '"%s"' % "-".join([core.hexdigest() for core in core_hashes])
         self.etags_by_media_type[media_type] = etag
@@ -945,8 +949,10 @@ class ReadWriteResource(HTTPResource):
                 # re-raise the exception, let the publisher look up
                 # the view, and hopefully handle it better.  Note the careful
                 # reraising that ensures the original traceback is preserved.
-                six.reraise(
-                    exception_info[0], exception_info[1], exception_info[2])
+                if six.PY3:
+                    six.raise_from(exception_info[1], None)
+                else:
+                    six.reraise(*exception_info)
 
             self.request.response.setStatus(ws_view.status)
             if ws_view.status // 100 == 4:
