@@ -11,7 +11,7 @@ __all__ = [
     ]
 
 
-from cStringIO import StringIO
+from six import StringIO
 import zope.proxy
 
 from zope.security.checker import getChecker, Checker, CheckerPublic, Proxy
@@ -20,7 +20,7 @@ from zope.security.checker import getChecker, Checker, CheckerPublic, Proxy
 def typename(obj):
     """Return the typename of an object."""
     t = type(obj)
-    if t.__module__ == '__builtin__':
+    if t.__module__ in {'__builtin__', 'builtins'}:
         return t.__name__
     else:
         return "%s.%s" % (t.__module__, t.__name__)
