@@ -2,6 +2,8 @@
 
 """A simple security policy for the LAZR example web service."""
 
+from __future__ import absolute_import, print_function
+
 __metaclass__ = type
 __all__ = [
     'CookbookWebServiceSecurityPolicy',
@@ -9,7 +11,7 @@ __all__ = [
 
 from zope.security.simplepolicies import PermissiveSecurityPolicy
 from zope.security.proxy import removeSecurityProxy
-from lazr.restful.example.base.interfaces import ICookbook, IRecipe
+from lazr.restful.example.base.interfaces import IRecipe
 
 
 class CookbookWebServiceSecurityPolicy(PermissiveSecurityPolicy):
