@@ -1,23 +1,26 @@
+from __future__ import absolute_import, print_function
 
-from zope.component import adapts
-from zope.interface import implements, Interface
+from zope.component import adapter
+from zope.interface import implementer, Interface
 from zope.schema import List, Text
 
 from lazr.restful.example.base.interfaces import IRecipe
 
 from lazr.restful.declarations import (
-    export_as_webservice_entry, exported)
+    exported,
+    exported_as_webservice_entry,
+    )
 
 
+@exported_as_webservice_entry(contributes_to=[IRecipe])
 class IHasComments(Interface):
-    export_as_webservice_entry(contributes_to=[IRecipe])
     comments = exported(
         List(title=u'Comments made by users', value_type=Text()))
 
 
+@implementer(IHasComments)
+@adapter(IRecipe)
 class RecipeToHasCommentsAdapter:
-    implements(IHasComments)
-    adapts(IRecipe)
 
     def __init__(self, recipe):
         self.recipe = recipe
