@@ -12,12 +12,16 @@ import doctest
 from pkg_resources import resource_filename
 
 from van.testing.layer import zcml_layer, wsgi_intercept_layer
+from zope.testing import renormalizing
 
 from lazr.restful.example.base.tests.test_integration import (
     CookbookWebServiceTestPublication, DOCTEST_FLAGS)
 from lazr.restful.testing.webservice import WebServiceApplication
 
 
+checker = renormalizing.OutputChecker()
+
+
 class FunctionalLayer:
     allow_teardown = False
     zcml = os.path.abspath(resource_filename(
@@ -40,7 +44,8 @@ def load_tests(loader, tests, pattern):
         'print_function': print_function,
         }
     suite = doctest.DocFileSuite(
-        *doctest_files, optionflags=DOCTEST_FLAGS, globs=globs)
+        *doctest_files, optionflags=DOCTEST_FLAGS, globs=globs,
+        checker=checker)
     suite.layer = WSGILayer
     tests.addTest(suite)
     return tests
