@@ -2,6 +2,8 @@
 #
 """A class for storing resources where they can be seen by a template."""
 
+from __future__ import absolute_import, print_function
+
 __metaclass__ = type
 
 __all__ = [
@@ -11,14 +13,14 @@ __all__ = [
 from lazr.restful.interfaces import (
     IJSONRequestCache, LAZR_WEBSERVICE_NS)
 
-from zope.component import adapts
-from zope.interface import implements
+from zope.component import adapter
+from zope.interface import implementer
 from zope.publisher.interfaces import IApplicationRequest
 
+@implementer(IJSONRequestCache)
+@adapter(IApplicationRequest)
 class JSONRequestCache:
     """Default implementation for `IJSONRequestCache`."""
-    implements(IJSONRequestCache)
-    adapts(IApplicationRequest)
 
     LAZR_OBJECT_JSON_CACHE = ("%s.object-json-cache"
                               % LAZR_WEBSERVICE_NS)
