@@ -25,10 +25,11 @@ __all__ = [
     ]
 
 from datetime import datetime
+from io import BytesIO
 import pytz
-from StringIO import StringIO
 
 import simplejson
+import six
 from six.moves.urllib.parse import unquote
 
 from zope.datetime import (
@@ -82,13 +83,14 @@ class URLDereferencingMixin:
         else:
             site_protocol = 'http'
             default_port = '80'
-        request_host = self.request.get('HTTP_HOST', 'localhost')
-        if ':' in request_host:
-            request_host, request_port = request_host.split(':', 2)
+        full_request_host = self.request.get('HTTP_HOST', 'localhost')
+        if ':' in full_request_host:
+            request_host, request_port = full_request_host.split(':', 2)
         else:
+            request_host = full_request_host
             request_port = default_port
 
-        if not isinstance(url, basestring):
+        if not isinstance(url, six.string_types):
             raise ValueError(u"got '%s', expected string: %r" % (
                 type(url).__name__, url))
         if url.startswith('/'):
@@ -116,11 +118,35 @@ class URLDereferencingMixin:
         path_parts = [unquote(part) for part in path.split('/')]
         path_parts.pop(0)
         path_parts.reverse()
-        request = config.createRequest(StringIO(), {'PATH_INFO': path})
+        environ = {'PATH_INFO': path, 'HTTP_HOST': full_request_host}
+        server_url = self.request.get('SERVER_URL')
+        if server_url is not None:
+            environ['SERVER_URL'] = server_url
+        else:
+            environ['SERVER_URL'] = '%s://%s' % (
+                site_protocol, full_request_host)
+        request = config.createRequest(BytesIO(), environ)
         request.setTraversalStack(path_parts)
         root = request.publication.getApplication(self.request)
         return request.traverse(root)
 
+    def dereference_url_as_object(self, url):
+        """Look up a data model object in the web service by URL."""
+        try:
+            resource = self.dereference_url(url)
+        except NotFound:
+            # The URL doesn't correspond to any real object.
+            raise ValueError(u'No such object "%s".' % url)
+        except InvalidURIError:
+            raise ValueError(u'"%s" is not a valid URI.' % url)
+        # We looked up the URL and got the thing at the other end of
+        # the URL: a resource. But internally, a resource isn't a
+        # valid value for any schema field. Instead we want the object
+        # that serves as a resource's context. Any time we want to get
+        # to the object underlying a resource, we need to strip its
+        # security proxy.
+        return removeSecurityProxy(resource).context
+
 
 @implementer(IFieldMarshaller)
 class SimpleFieldMarshaller:
@@ -156,10 +182,10 @@ class SimpleFieldMarshaller:
         if value != '':
             try:
                 v = value
-                if isinstance(v, str):
+                if isinstance(v, bytes):
                     v = v.decode('utf8') # assume utf8
-                elif not isinstance(v, unicode):
-                    v = unicode(v)
+                elif not isinstance(v, six.text_type):
+                    v = six.text_type(v)
                 value = simplejson.loads(v)
             except (ValueError, TypeError):
                 # Pass the value as is. This saves client from having to encode
@@ -245,7 +271,7 @@ class FloatFieldMarshaller(SimpleFieldMarshaller):
 class BytesFieldMarshaller(SimpleFieldMarshaller):
     """FieldMarshaller for IBytes field."""
 
-    _type = str
+    _type = bytes
     _type_error_message = 'not a string: %r'
 
     @property
@@ -273,8 +299,8 @@ class BytesFieldMarshaller(SimpleFieldMarshaller):
         if safe_hasattr(value, 'seek'):
             value.seek(0)
             value = value.read()
-        elif not isinstance(value, basestring):
-            value = str(value)
+        elif not isinstance(value, (bytes, six.text_type)):
+            value = bytes(value)
         else:
             # Leave string conversion to _marshall_from_json_data.
             pass
@@ -285,7 +311,7 @@ class BytesFieldMarshaller(SimpleFieldMarshaller):
 
         Convert all strings to byte strings.
         """
-        if isinstance(value, unicode):
+        if isinstance(value, six.text_type):
             value = value.encode('utf-8')
         return super(
             BytesFieldMarshaller, self)._marshall_from_json_data(value)
@@ -294,7 +320,7 @@ class BytesFieldMarshaller(SimpleFieldMarshaller):
 class TextFieldMarshaller(SimpleFieldMarshaller):
     """FieldMarshaller for IText fields."""
 
-    _type = unicode
+    _type = six.text_type
     _type_error_message = 'not a unicode string: %r'
 
     def _marshall_from_request(self, value):
@@ -302,7 +328,7 @@ class TextFieldMarshaller(SimpleFieldMarshaller):
 
         Converts the value to unicode.
         """
-        value = unicode(value)
+        value = six.text_type(value)
         return super(TextFieldMarshaller, self)._marshall_from_request(value)
 
 
@@ -344,7 +370,8 @@ class TokenizedVocabularyFieldMarshaller(FixedVocabularyFieldMarshaller):
         Looks up the value as a token in the vocabulary.
         """
         try:
-            return self.field.vocabulary.getTermByToken(str(value)).value
+            return self.field.vocabulary.getTermByToken(
+                six.text_type(value)).value
         except LookupError:
             raise ValueError(u"%r isn't a valid token" % value)
 
@@ -633,17 +660,4 @@ class ObjectLookupFieldMarshaller(SimpleFieldMarshaller,
 
         Look up the data model object by URL.
         """
-        try:
-            resource = self.dereference_url(value)
-        except NotFound:
-            # The URL doesn't correspond to any real object.
-            raise ValueError(u'No such object "%s".' % value)
-        except InvalidURIError:
-            raise ValueError(u'"%s" is not a valid URI.' % value)
-        # We looked up the URL and got the thing at the other end of
-        # the URL: a resource. But internally, a resource isn't a
-        # valid value for any schema field. Instead we want the object
-        # that serves as a resource's context. Any time we want to get
-        # to the object underlying a resource, we need to strip its
-        # security proxy.
-        return removeSecurityProxy(resource).context
+        return self.dereference_url_as_object(value)
