@@ -13,6 +13,7 @@ __all__ = [
     'FakeResponse',
     'IGenericEntry',
     'IGenericCollection',
+    'pprint_collection',
     'pprint_entry',
     'simple_renderer',
     'WebServiceTestCase',
@@ -174,6 +175,24 @@ class FakeRequest:
         return default
 
 
+def pformat_value(value):
+    """Pretty-format a single value.
+
+    This is similar to `repr()`, but for doctest compatibility we format
+    text and bytes in a way that looks the same on both Python 2 and 3.
+    JSON strings are always Unicode, never bytes, so this doesn't introduce
+    ambiguity.
+    """
+    if isinstance(value, six.text_type):
+        value = value.encode('unicode_escape').decode('ASCII')
+        if "'" in value and '"' not in value:
+            return '"%s"' % value
+        else:
+            return "'%s'" % value.replace("'", "\\'")
+    else:
+        return repr(value)
+
+
 def pprint_entry(json_body):
     """Pretty-print a webservice entry JSON representation.
 
@@ -182,7 +201,7 @@ def pprint_entry(json_body):
     """
     for key, value in sorted(json_body.items()):
         if key != 'http_etag':
-            print('%s: %r' % (key, value))
+            print('%s: %s' % (key, pformat_value(value)))
 
 
 def pprint_collection(json_body):
@@ -191,7 +210,7 @@ def pprint_collection(json_body):
         if key == 'total_size_link':
             continue
         if key != 'entries':
-            print('%s: %r' % (key, value))
+            print('%s: %s' % (key, pformat_value(value)))
     print('---')
     for entry in json_body['entries']:
         pprint_entry(entry)
@@ -392,6 +411,7 @@ class WebServiceAjaxCaller(WebServiceCaller):
         return (config.path_override + '/' + default_version)
 
 
+@six.python_2_unicode_compatible
 class WebServiceResponseWrapper(ProxyBase):
     """A response from the web service with easy access to the JSON body."""
 
@@ -403,21 +423,32 @@ class WebServiceResponseWrapper(ProxyBase):
         return self.getheader(key, default)
 
     def __str__(self):
+        if sys.version_info[0] >= 3:
+            # Reconstructing the original list of headers seems to involve a
+            # bit more effort on Python 3.
+            def normalize(key):
+                """Normalize an HTTP header name to title case."""
+                return '-'.join(s.title() for s in key.split('-'))
+
+            headers = [
+                '%s: %s\n' % (normalize(key), value)
+                for key, value in self.msg.items()]
+        else:
+            headers = self.msg.headers
+        body = self.body.decode('UTF-8')
         return 'HTTP/1.1 %s %s\n%s\n%s' % (
-            self.status, self.reason, ''.join(self.msg.headers), self.body)
+            self.status, self.reason, ''.join(headers), body)
 
     def jsonBody(self):
         """Return the body of the web service request as a JSON document."""
+        body = six.ensure_text(self.body)
         try:
-            json = simplejson.loads(six.ensure_text(self.body))
-            if isinstance(json, list):
-                json = sorted(json)
-            return json
+            return simplejson.loads(body)
         except ValueError:
             # Return a useful ValueError that displays the problematic
             # string, instead of one that just says the string wasn't
             # JSON.
-            raise ValueError(self.body)
+            raise ValueError(body)
 
 
 @implementer(IWebServiceConfiguration)
