@@ -1,6 +1,8 @@
 # Copyright 2008 Canonical Ltd.  All rights reserved.
 """Tests for ETag generation."""
 
+from __future__ import absolute_import, print_function
+
 __metaclass__ = type
 
 import unittest
@@ -12,7 +14,6 @@ from lazr.restful.testing.helpers import TestWebServiceConfiguration
 from lazr.restful.testing.webservice import create_web_service_request
 from lazr.restful._resource import (
     EntryFieldResource,
-    EntryResource,
     HTTPResource,
     ServiceRootResource,
     make_entry_etag_cores,
@@ -28,7 +29,7 @@ class TestEntryResourceETags(unittest.TestCase):
     def test_no_field_details(self):
         # If make_entry_etag_cores is given no field details (because no
         # fields exist), the resulting cores empty strings.
-        self.assertEquals(make_entry_etag_cores([]), ['', ''])
+        self.assertEqual(make_entry_etag_cores([]), ['', ''])
 
     def test_writable_fields(self):
         # If there are writable fields, their values are incorporated into the
@@ -41,7 +42,7 @@ class TestEntryResourceETags(unittest.TestCase):
              {'writable': True,
               'value': 'second'}),
             ]
-        self.assertEquals(
+        self.assertEqual(
             make_entry_etag_cores(field_details), ['', 'first\0second'])
 
     def test_unchanging_fields(self):
@@ -53,7 +54,7 @@ class TestEntryResourceETags(unittest.TestCase):
              {'writable': False,
               'value': 'the value'}),
             ]
-        self.assertEquals(
+        self.assertEqual(
             make_entry_etag_cores(field_details),
             ['the value', ''])
 
@@ -74,7 +75,7 @@ class TestEntryResourceETags(unittest.TestCase):
              {'writable': False,
               'value': 'second-not-writable'}),
             ]
-        self.assertEquals(
+        self.assertEqual(
             make_entry_etag_cores(field_details),
             ['first-not-writable\x00second-not-writable',
              'first-writable\x00second-writable'])
@@ -86,16 +87,7 @@ class TestHTTPResourceETags(unittest.TestCase):
         # The HTTPResource class implements a do-nothing _getETagCores in order to
         # be conservative (because it's not aware of the nature of all possible
         # subclasses).
-        self.assertEquals(HTTPResource(None, None)._getETagCores(), None)
-
-
-class TestHTTPResourceETags(unittest.TestCase):
-
-    def test_getETag_is_a_noop(self):
-        # The HTTPResource class implements a do-nothing _getETagCores in order to
-        # be conservative (because it's not aware of the nature of all possible
-        # subclasses).
-        self.assertEquals(HTTPResource(None, None)._getETagCores(), None)
+        self.assertEqual(HTTPResource(None, None)._getETagCores(), None)
 
 
 class FauxEntryField:
@@ -189,6 +181,24 @@ class ServiceRootResourceTests(unittest.TestCase):
         # The cores should be different.
         self.assertNotEqual(first_cores, second_cores)
 
+    def test_wadl_etags_change_with_revno(self):
+        # The ETag should change if the revision (not the version) of the
+        # web service changes.  (At one point this was incorrectly
+        # special-cased to exclude the core, and so included no revision
+        # information.)
+
+        # Find the ETag generated with a given revision...
+        self.config.code_revision = u'42'
+        first_etag = self.resource.getETag(self.resource.WADL_TYPE)
+
+        # ...and find the ETag generated with a different revision.
+        self.resource.etags_by_media_type = {}
+        self.config.code_revision = u'99'
+        second_etag = self.resource.getETag(self.resource.WADL_TYPE)
+
+        # The ETags should be different.
+        self.assertNotEqual(first_etag, second_etag)
+
 
 class TestableHTTPResource(HTTPResource):
     """A HTTPResource that lest us set the ETags from the outside."""
@@ -214,8 +224,8 @@ class TestConditionalGet(unittest.TestCase):
         # a conditional GET should return "Not Modified" (304).
         self.resource.incoming_etags = ['1', '2', '3']
         self.resource.existing_etag = '2'
-        self.assertEquals(self.resource.handleConditionalGET(), None)
-        self.assertEquals(self.request.response.getStatus(), 304)
+        self.assertEqual(self.resource.handleConditionalGET(), None)
+        self.assertEqual(self.request.response.getStatus(), 304)
 
     def test_etags_differ(self):
         # If none of the ETags present in an incoming request is the same as
@@ -224,7 +234,7 @@ class TestConditionalGet(unittest.TestCase):
         # being returned.
         self.resource.incoming_etags = ['1', '2', '3']
         self.resource.existing_etag = '99'
-        self.assertNotEquals(self.resource.handleConditionalGET(), None)
+        self.assertNotEqual(self.resource.handleConditionalGET(), None)
 
 
 class TestConditionalWrite(unittest.TestCase):
@@ -241,7 +251,7 @@ class TestConditionalWrite(unittest.TestCase):
         # the write should be applied.
         self.resource.incoming_etags = ['1', '2', '3']
         self.resource.existing_etag = '2'
-        self.assertNotEquals(self.resource.handleConditionalWrite(), None)
+        self.assertNotEqual(self.resource.handleConditionalWrite(), None)
 
     def test_etags_differ(self):
         # If one of the ETags present in an incoming request is the same as
@@ -249,5 +259,5 @@ class TestConditionalWrite(unittest.TestCase):
         # the write should fail.
         self.resource.incoming_etags = ['1', '2', '3']
         self.resource.existing_etag = '99'
-        self.assertEquals(self.resource.handleConditionalWrite(), None)
-        self.assertEquals(self.request.response.getStatus(), 412)
+        self.assertEqual(self.resource.handleConditionalWrite(), None)
+        self.assertEqual(self.request.response.getStatus(), 412)
