@@ -1,5 +1,7 @@
 """A WSGI application for a lazr.restful web service."""
 
+from __future__ import absolute_import, print_function
+
 __metaclass__ = type
 __all__ = [
     'BaseWSGIWebServiceConfiguration',
@@ -30,8 +32,7 @@ class WSGIApplication:
 
     def __iter__(self):
         environ = self.environ
-        # Create the request based on the HTTP method used.
-        method = environ.get('REQUEST_METHOD', 'GET').upper()
+        # Create the request.
         service_root = getUtility(IServiceRootResource)
         request = self.request_class(environ['wsgi.input'], environ)
         request.setPublication(self.publication_class(service_root))
